/*
   	This is a comment block.  Anything inside the symbols at 
   	the beginning and end of this comment will not executed as
   	 part of your program
 	
 	
 */

/*=======================include libraries=======================*/
#include <stdio.h>
#include <math.h>
#include <GL/glut.h>

#ifdef WIN32
    extern "C" {
    #include  "texture.h"
    }
#endif

#ifndef WIN32
    #include "texture.h"
#endif

/*=============================================================*/

/*+++++++++++++++++++++++function prototypes+++++++++++++++++++*/
char 	graphic( double );
void Idle(void);
void InitLists( void ); /* initializes display lists*/
void InitGraphics( void ); /* initializes graphics*/
void Display( void ); /* good old friend the display function callback*/

void loadImageTexture(void);
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


/*<<<<<<<<<<<<<<<<<<<<<< global variables <<<<<<<<<<<<<<<<<<<<<<<*/

#define MAXGRID 63

int Debug; /* ON means print debug info */
int GrWindow; /* window id for graphics window */
int Objectlist; /* object display lists */
const char *WINDOWTITLE = { " Texture mapping  example " };
const int INIT_WINDOW_SIZE = { 800 };/* initial window size: */
const float BACKCOLOR[ ] = { 0., 0., 0., 0. };/* window background color (rgba): */
const int ORTHO =   0;
const int PERSP    =    1;/* which button: */
int WhichProjection=PERSP; /* ORTHO or PERSP */
float Xrot = - 80 , Yrot = 30; /* rotation angles in degrees */
float RotMatrix[4][4]; /* set by glui rotation widget */
float Scale,Scale2; /* scaling factors */
float TransXYZ[3]; /* set by glui translation widgets */
const float MINSCALE = { 0.05f };/* minimum allowable scale factor: */
float   thetax,thetay,thetaz;


int texWidth, texHeight;
GLubyte *texData;
char *texFilename1  = "diablogtr32.sgi";
GLuint texId1;
float texCoords[MAXGRID][MAXGRID][2];

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>*/






int main( int argc, char **argv )
{
    /* Initalize OpenGL Toolkit */
    glutInit( &argc, argv );

    /* setup all the graphics stuff, including registering event callbacks with glut*/
    InitGraphics(); 

    /* create the display structures that will not change: */
    InitLists(); 
    loadImageTexture();
    glutMainLoop(); 

    return 0;
}

/**
** initialize the glut and OpenGL libraries:
** also setup display lists and callback functions
**/
void
InitGraphics( void )
{

    thetax = 0.;
    thetay = 0.;
    thetaz = 0.;
    
    Scale = .3f;
    Scale2 = 0.0; /* because add 1. to it in Display() */

    TransXYZ[0] = TransXYZ[1] = TransXYZ[2] = 0.;
    RotMatrix[0][1] = RotMatrix[0][2] = RotMatrix[0][3] = 0.;
    RotMatrix[1][0] = RotMatrix[1][2] = RotMatrix[1][3] = 0.;
    RotMatrix[2][0] = RotMatrix[2][1] = RotMatrix[2][3] = 0.;
    RotMatrix[3][0] = RotMatrix[3][1] = RotMatrix[3][3] = 0.;
    RotMatrix[0][0] = RotMatrix[1][1] = RotMatrix[2][2] = RotMatrix[3][3] = 1.;

 //   tex = ReadFileBMP("apple.BMP");

    if( Debug )
        fprintf( stderr, "InitGraphics\n" ); 

    /* setup the display mode: */
    /* ( *must* be done before call to glutCreateWindow() ) */
    /* ask for color, double-buffering, and z-buffering: */


    glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH ); 

    /* set the initial window configuration: */


    glutInitWindowSize( INIT_WINDOW_SIZE, INIT_WINDOW_SIZE ); 

    /* open the window and set its title: */


    GrWindow = glutCreateWindow( WINDOWTITLE );


    glutSetWindowTitle( WINDOWTITLE );


    glutPositionWindow(400, 40);


    /* setup the clear values: */
    glClearColor( BACKCOLOR[0], BACKCOLOR[1], BACKCOLOR[2], BACKCOLOR[3] ); 

    /* setup the callback routines: */

    /* DisplayFunc -- redrawthe window */
    /* ReshapeFunc -- handle the user resizing the window */
    /* KeyboardFunc -- handle a keyboard input */
    /* MouseFunc -- handle the mouse button going down or up */
    /* MotionFunc -- handle the mouse moving with a button down */
    /* PassiveMotionFunc -- handle the mouse moving with a button up*/
    /* VisibilityFunc -- handle a change in window visibility */
    /* EntryFunc -- handle the cursor entering or leaving the window */
    /* SpecialFunc -- handle special keys on the keyboard */
    /* SpaceballMotionFunc -- handle spaceball translation */
    /* SpaceballRotateFunc -- handle spaceball rotation */
    /* SpaceballButtonFunc -- handle spaceball button hits */
    /* ButtonBoxFunc -- handle button box hits */
    /* DialsFunc -- handle dial rotations */
    /* TabletMotionFunc -- handle digitizing tablet motion */
    /* TabletButtonFunc -- handle digitizing tablet button hits */
    /* MenuStateFunc -- declare when a pop-up menu is in use */
    /* IdleFunc -- what to do when nothing else is going on */
    /* TimerFunc -- trigger something to happen every so often */


    glutSetWindow( GrWindow );
    glutDisplayFunc( Display );
    glutReshapeFunc( NULL );
    glutKeyboardFunc( NULL );
    glutMouseFunc( NULL );
    glutMotionFunc( NULL );
    glutPassiveMotionFunc( NULL );
    //glutVisibilityFunc( Visibility );
    glutEntryFunc( NULL );
    glutSpecialFunc( NULL );
    glutSpaceballMotionFunc( NULL );
    glutSpaceballRotateFunc( NULL );
    glutSpaceballButtonFunc( NULL );
    glutButtonBoxFunc( NULL );
    glutDialsFunc( NULL );
    glutTabletMotionFunc( NULL );
    glutTabletButtonFunc( NULL );
    glutMenuStateFunc( NULL );
    glutIdleFunc( Idle );

    /*But we will set our timer function here:*/
    //glutTimerFunc( 33, myTimer, 1 );



}

void InitLists( void )
{


    if( Debug )
        fprintf( stderr, "InitLists\n" );

}

void Display(void)
{


    int dx, dy, d; /* viewport dimensions */
    int xl, yb; /* lower-left corner of viewport */
    float scale2; /* real glui scale factor */
    
    if( Debug )
    { 
        fprintf( stderr, "Display\n" );
    }    


    /* set which window we want to do the graphics into: */ 
    glutSetWindow( GrWindow ); /* erase the background: */
    glDrawBuffer( GL_BACK );
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    glEnable( GL_DEPTH_TEST );

    /* specify shading to be smooth: */
    glShadeModel( GL_SMOOTH ); 


    /* set the viewport to a square centered in the window, basically doing the resize functions here: */


    dx = glutGet( GLUT_WINDOW_WIDTH );
    dy = glutGet( GLUT_WINDOW_HEIGHT );
    d = dx < dy ? dx : dy; /* minimum dimension */
    xl = ( dx - d ) / 2;
    yb = ( dy - d ) / 2;


    glViewport( xl, yb, d, d ); /* set the viewing volume: */


    /* remember that the eye is at the origin looking in -Z */
    /* remember that the Z values are actually */
    /* given as DISTANCES IN FRONT OF THE EYE */
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();


    if( WhichProjection == ORTHO )


    glOrtho( -3., 3., -3., 3., 0.1, 1000. );



    else

    gluPerspective( 90., 1., 0.1, 1000. ); /* place the object into the viewing volume: */




    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
    gluLookAt( 0., 0., 6., 0., 0., 0., 0., 1., 0. );


    /* rotate the scene: */
    glRotatef( Yrot, 0., 1., 0. );
    glRotatef( Xrot, 1., 0., 0. );
    glMultMatrixf( (const GLfloat *) RotMatrix );



    /* scale the scene: */
    glScalef( Scale, Scale, Scale );
    scale2 = 1. + Scale2; /* because glui translation starts at 0. */


    if( scale2 < MINSCALE )

    scale2 = MINSCALE;



    glScalef( scale2, scale2, scale2 ); /* possibly draw the axes: */




    /* translate the object: */


    /* note the minus sign on the z value */
    /* this is to make the appearance of the glui z translate */
    /* widget more intuitively match the translate behavior */
    //glTranslatef( TransXYZ[0], TransXYZ[1], -TransXYZ[2] );

/*===========================*/
/*   Texture application code       */
/*===========================*/
    glEnable(GL_TEXTURE_2D);
    glPushMatrix();
    
      glRotatef(thetax,1.,0,0);
      glRotatef(thetay,0,1.,0);
      glRotatef(thetaz,0,0,1.);

    glBegin(GL_QUADS);
      glNormal3f(0.,0.,1.);
      glTexCoord2f(0.,0.);
      glVertex3f(-20.,-10.,-10.);
    
      glTexCoord2f(3.,0.);
      glVertex3f(20.,-10.,-10.);
    
      glTexCoord2f(3.,3.);
      glVertex3f(20.,10.,-10.);
    
      glTexCoord2f(0.,3.);
      glVertex3f(-20.,10.,-10.);
    glEnd();

    glPopMatrix();
    glDisable(GL_TEXTURE_2D);
/*===========================*/
/*                                                      */
/*===========================*/


    //put your display code here!
    GLfloat diff[] = {1.,1.,1.};
    GLfloat spec[] = {1.,1.,1.};
    GLfloat pos[] = {3.,-5.,5.};

    glLightfv(GL_LIGHT0,GL_POSITION,pos);
    glLightfv(GL_LIGHT0,GL_SPECULAR,spec);
    glLightfv(GL_LIGHT0,GL_DIFFUSE,diff);
    glEnable(GL_LIGHT0);
    glEnable(GL_LIGHTING);
    //    glEnable(GL_NORMALIZE);
    glPushMatrix();
     glRotatef(thetax,1.,0,0);
     glRotatef(thetay,0,1.,0);
     glRotatef(thetaz,0,0,1.);
     glutSolidCube(4.);
    glPopMatrix();


    /* swap the double-buffered framebuffers: */
    glutSwapBuffers(); 

    /* be sure the graphics buffer has been sent: */
    glFlush();

}

void Idle( void )
{
    thetax += 0.09f;
    thetay += 0.02f; 
    thetaz = 0.;
    glutSetWindow( GrWindow );
    glutPostRedisplay();
}




void loadImageTexture(void)
{
    
    glGenTextures(1,&texId1); /* create a texture object */
    
    /* bind the texture - use to switch between current
     textures, or to initially create a bound texture object */
    glBindTexture(GL_TEXTURE_2D, texId1); 
    
    /* load the texture data from an SGI  .rgb file  */
    imgLoad(texFilename1, 0, 0, &texWidth, &texHeight, &texData);
    
    
    /* the options commented out would cause the texture to clamp at its 
    boundaries.  In other words, it would not repeat if the object it is bound 
    to is larger than (S,T) = (1,1) */
   // glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
   // glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
   
   /* these options set how the image is filtered, since it will likely not be a 
   pixel by pixel size match   */
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    
    /* The following function loads the texture data into the texture object, 
    with various options */
    glTexImage2D(GL_TEXTURE_2D, 0, 4, texWidth, texHeight, 0, GL_RGBA, 
                                                                                GL_UNSIGNED_BYTE, texData );
                                                                                
                                                                                
}
